@@ -18,7 +18,6 @@
 from contextlib import closing
 from datetime import datetime
 from typing import Any, Optional
-from urllib.parse import quote_plus, urlunsplit
 
 from sqlalchemy import create_engine
 
@@ -57,7 +56,6 @@ class DbApiHook(BaseHook):
     :param schema: Optional DB schema that overrides the schema specified in the connection. Make sure that
         if you change the schema parameter value in the constructor of the derived Hook, such change
         should be done before calling the ``DBApiHook.__init__()``.
-    :type schema: Optional[str]
     """
 
     # Override to provide the connection name.
@@ -68,6 +66,8 @@ class DbApiHook(BaseHook):
     supports_autocommit = False
     # Override with the object that exposes the connect method
     connector = None  # type: Optional[ConnectorProtocol]
+    # Override with db-specific query to check connection
+    _test_connection_sql = "select 1"
 
     def __init__(self, *args, schema: Optional[str] = None, **kwargs):
         super().__init__()
@@ -97,14 +97,8 @@ class DbApiHook(BaseHook):
         :return: the extracted uri.
         """
         conn = self.get_connection(getattr(self, self.conn_name_attr))
-        login = ''
-        if conn.login:
-            login = f'{quote_plus(conn.login)}:{quote_plus(conn.password)}@'
-        host = conn.host
-        if conn.port is not None:
-            host += f':{conn.port}'
-        schema = self.__schema or conn.schema or ''
-        return urlunsplit((conn.conn_type, f'{login}{host}', schema, '', ''))
+        conn.schema = self.__schema or conn.schema
+        return conn.get_uri()
 
     def get_sqlalchemy_engine(self, engine_kwargs=None):
         """
@@ -123,11 +117,8 @@ class DbApiHook(BaseHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         :param kwargs: (optional) passed into pandas.io.sql.read_sql method
-        :type kwargs: dict
         """
         try:
             from pandas.io import sql as psql
@@ -137,15 +128,31 @@ class DbApiHook(BaseHook):
         with closing(self.get_conn()) as conn:
             return psql.read_sql(sql, con=conn, params=parameters, **kwargs)
 
+    def get_pandas_df_by_chunks(self, sql, parameters=None, *, chunksize, **kwargs):
+        """
+        Executes the sql and returns a generator
+
+        :param sql: the sql statement to be executed (str) or a list of
+            sql statements to execute
+        :param parameters: The parameters to render the SQL query with
+        :param chunksize: number of rows to include in  each chunk
+        :param kwargs: (optional) passed into pandas.io.sql.read_sql method
+        """
+        try:
+            from pandas.io import sql as psql
+        except ImportError:
+            raise Exception("pandas library not installed, run: pip install 'apache-airflow[pandas]'.")
+
+        with closing(self.get_conn()) as conn:
+            yield from psql.read_sql(sql, con=conn, params=parameters, chunksize=chunksize, **kwargs)
+
     def get_records(self, sql, parameters=None):
         """
         Executes the sql and returns a set of records.
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         """
         with closing(self.get_conn()) as conn:
             with closing(conn.cursor()) as cur:
@@ -161,9 +168,7 @@ class DbApiHook(BaseHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         """
         with closing(self.get_conn()) as conn:
             with closing(conn.cursor()) as cur:
@@ -181,20 +186,21 @@ class DbApiHook(BaseHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param autocommit: What to set the connection's autocommit setting to
             before executing the query.
-        :type autocommit: bool
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         :param handler: The result handler which is called with the result of each statement.
-        :type handler: callable
         :return: query results if handler was provided.
         """
         scalar = isinstance(sql, str)
         if scalar:
             sql = [sql]
 
+        if sql:
+            self.log.debug("Executing %d statements", len(sql))
+        else:
+            raise ValueError("List of SQL statements is empty")
+
         with closing(self.get_conn()) as conn:
             if self.supports_autocommit:
                 self.set_autocommit(conn, autocommit)
@@ -249,7 +255,6 @@ class DbApiHook(BaseHook):
         does not support autocommit.
 
         :param conn: Connection to get autocommit setting from.
-        :type conn: connection object.
         :return: connection autocommit setting.
         :rtype: bool
         """
@@ -266,13 +271,9 @@ class DbApiHook(BaseHook):
         The REPLACE variant is specific to MySQL syntax.
 
         :param table: Name of the target table
-        :type table: str
         :param values: The row to insert into the table
-        :type values: tuple of cell values
         :param target_fields: The names of the columns to fill in the table
-        :type target_fields: iterable of strings
         :param replace: Whether to replace instead of insert
-        :type replace: bool
         :return: The generated INSERT or REPLACE SQL statement
         :rtype: str
         """
@@ -299,16 +300,11 @@ class DbApiHook(BaseHook):
         a new transaction is created every commit_every rows
 
         :param table: Name of the target table
-        :type table: str
         :param rows: The rows to insert into the table
-        :type rows: iterable of tuples
         :param target_fields: The names of the columns to fill in the table
-        :type target_fields: iterable of strings
         :param commit_every: The maximum number of rows to insert in one
             transaction. Set to 0 to insert all rows in one transaction.
-        :type commit_every: int
         :param replace: Whether to replace instead of insert
-        :type replace: bool
         """
         i = 0
         with closing(self.get_conn()) as conn:
@@ -339,9 +335,7 @@ class DbApiHook(BaseHook):
         Returns the SQL literal of the cell as a string.
 
         :param cell: The cell to insert into the table
-        :type cell: object
         :param conn: The database connection
-        :type conn: connection object
         :return: The serialized cell
         :rtype: str
         """
@@ -356,9 +350,7 @@ class DbApiHook(BaseHook):
         Dumps a database table into a tab-delimited file
 
         :param table: The name of the source table
-        :type table: str
         :param tmp_file: The path of the target file
-        :type tmp_file: str
         """
         raise NotImplementedError()
 
@@ -367,22 +359,17 @@ class DbApiHook(BaseHook):
         Loads a tab-delimited file into a database table
 
         :param table: The name of the target table
-        :type table: str
         :param tmp_file: The path of the file to load into the table
-        :type tmp_file: str
         """
         raise NotImplementedError()
 
     def test_connection(self):
-        """Tests the connection by executing a select 1 query"""
+        """Tests the connection using db-specific query"""
         status, message = False, ''
         try:
-            with closing(self.get_conn()) as conn:
-                with closing(conn.cursor()) as cur:
-                    cur.execute("select 1")
-                    if cur.fetchone():
-                        status = True
-                        message = 'Connection successfully tested'
+            if self.get_first(self._test_connection_sql):
+                status = True
+                message = 'Connection successfully tested'
         except Exception as e:
             status = False
             message = str(e)
