@@ -84,6 +84,23 @@ class Pool(Base):
         """
         return Pool.get_pool(Pool.DEFAULT_POOL_NAME, session=session)
 
+    @staticmethod
+    @provide_session
+    def is_default_pool(id: int, session: Session = NEW_SESSION) -> bool:
+        """
+        Check id if is the default_pool.
+
+        :param id: pool id
+        :param session: SQLAlchemy ORM Session
+        :return: True if id is default_pool, otherwise False
+        """
+        return (
+            session.query(func.count(Pool.id))
+            .filter(Pool.id == id, Pool.pool == Pool.DEFAULT_POOL_NAME)
+            .scalar()
+            > 0
+        )
+
     @staticmethod
     @provide_session
     def create_or_update_pool(name: str, slots: int, description: str, session: Session = NEW_SESSION):
@@ -107,7 +124,7 @@ class Pool(Base):
     def delete_pool(name: str, session: Session = NEW_SESSION):
         """Delete pool by a given name."""
         if name == Pool.DEFAULT_POOL_NAME:
-            raise AirflowException("default_pool cannot be deleted")
+            raise AirflowException(f"{Pool.DEFAULT_POOL_NAME} cannot be deleted")
 
         pool = session.query(Pool).filter_by(pool=name).first()
         if pool is None:
@@ -174,11 +191,7 @@ class Pool(Base):
 
         # calculate open metric
         for pool_name, stats_dict in pools.items():
-            if stats_dict["total"] == -1:
-                # -1 means infinite
-                stats_dict["open"] = -1
-            else:
-                stats_dict["open"] = stats_dict["total"] - stats_dict["running"] - stats_dict["queued"]
+            stats_dict["open"] = stats_dict["total"] - stats_dict["running"] - stats_dict["queued"]
 
         return pools
 
@@ -196,7 +209,7 @@ class Pool(Base):
         }
 
     @provide_session
-    def occupied_slots(self, session: Session):
+    def occupied_slots(self, session: Session = NEW_SESSION):
         """
         Get the number of slots used by running/queued tasks at the moment.
 
@@ -214,7 +227,7 @@ class Pool(Base):
         )
 
     @provide_session
-    def running_slots(self, session: Session):
+    def running_slots(self, session: Session = NEW_SESSION):
         """
         Get the number of slots used by running tasks at the moment.
 
@@ -232,7 +245,7 @@ class Pool(Base):
         )
 
     @provide_session
-    def queued_slots(self, session: Session):
+    def queued_slots(self, session: Session = NEW_SESSION):
         """
         Get the number of slots used by queued tasks at the moment.
 
