@@ -20,10 +20,7 @@
 import warnings
 from typing import Dict, Optional, Sequence, Set, Tuple
 
-from flask import current_app, g
-from flask_appbuilder.security.sqla import models as sqla_models
-from flask_appbuilder.security.sqla.manager import SecurityManager
-from flask_appbuilder.security.sqla.models import PermissionView, Role, User
+from flask import g
 from sqlalchemy import or_
 from sqlalchemy.orm import joinedload
 
@@ -32,10 +29,11 @@ from airflow.models import DagBag, DagModel
 from airflow.security import permissions
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.session import provide_session
+from airflow.www.fab_security.sqla.manager import SecurityManager
+from airflow.www.fab_security.sqla.models import Permission, Resource, Role, User
 from airflow.www.utils import CustomSQLAInterface
 from airflow.www.views import (
-    CustomPermissionModelView,
-    CustomPermissionViewModelView,
+    ActionModelView,
     CustomResetMyPasswordView,
     CustomResetPasswordView,
     CustomRoleModelView,
@@ -46,7 +44,8 @@ from airflow.www.views import (
     CustomUserOIDModelView,
     CustomUserRemoteUserModelView,
     CustomUserStatsChartView,
-    CustomViewMenuModelView,
+    PermissionPairModelView,
+    ResourceModelView,
 )
 
 EXISTING_ROLES = {
@@ -58,7 +57,7 @@ EXISTING_ROLES = {
 }
 
 
-class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=too-many-public-methods
+class AirflowSecurityManager(SecurityManager, LoggingMixin):
     """Custom security manager, which introduces a permission model adapted to Airflow"""
 
     ###########################################################################
@@ -85,6 +84,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_XCOM),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_BROWSE_MENU),
+        (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_DAG_DEPENDENCIES),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_DAG_RUN),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_DOCS),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_DOCS_MENU),
@@ -113,6 +113,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
     OP_PERMISSIONS = [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_CONFIG),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_ADMIN_MENU),
+        (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_CONFIG),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_CONNECTION),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_POOL),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_VARIABLE),
@@ -137,15 +138,17 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
     ADMIN_PERMISSIONS = [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_TASK_RESCHEDULE),
         (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_TASK_RESCHEDULE),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_TRIGGER),
+        (permissions.ACTION_CAN_ACCESS_MENU, permissions.RESOURCE_TRIGGER),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_PASSWORD),
         (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_PASSWORD),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_ROLE),
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_ROLE),
     ]
 
-    # global view-menu for dag-level access
-    DAG_VMS = {permissions.RESOURCE_DAG}
-
-    READ_DAG_PERMS = {permissions.ACTION_CAN_READ}
-    DAG_PERMS = permissions.DAG_PERMS
+    # global resource for dag-level access
+    DAG_RESOURCES = {permissions.RESOURCE_DAG}
+    DAG_ACTIONS = permissions.DAG_ACTIONS
 
     ###########################################################################
     #                     DEFAULT ROLE CONFIGURATIONS
@@ -168,10 +171,10 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         },
     ]
 
-    permissionmodelview = CustomPermissionModelView
-    permissionviewmodelview = CustomPermissionViewModelView
+    actionmodelview = ActionModelView
+    permissionmodelview = PermissionPairModelView
     rolemodelview = CustomRoleModelView
-    viewmenumodelview = CustomViewMenuModelView
+    resourcemodelview = ResourceModelView
     userdbmodelview = CustomUserDBModelView
     resetmypasswordview = CustomResetMyPasswordView
     resetpasswordview = CustomResetPasswordView
@@ -197,9 +200,19 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
             view.datamodel = CustomSQLAInterface(view.datamodel.obj)
         self.perms = None
 
+    def _get_root_dag_id(self, dag_id):
+        if '.' in dag_id:
+            dm = (
+                self.get_session.query(DagModel.dag_id, DagModel.root_dag_id)
+                .filter(DagModel.dag_id == dag_id)
+                .first()
+            )
+            return dm.root_dag_id or dm.dag_id
+        return dag_id
+
     def init_role(self, role_name, perms):
         """
-        Initialize the role with the permissions and related view-menus.
+        Initialize the role with actions and related resources.
         :param role_name:
         :param perms:
         :return:
@@ -214,26 +227,20 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
     def bulk_sync_roles(self, roles):
         """Sync the provided roles and permissions."""
         existing_roles = self._get_all_roles_with_permissions()
-        pvs = self._get_all_non_dag_permissionviews()
+        non_dag_perms = self._get_all_non_dag_permissions()
 
         for config in roles:
             role_name = config['role']
             perms = config['perms']
             role = existing_roles.get(role_name) or self.add_role(role_name)
 
-            for perm_name, view_name in perms:
-                perm_view = pvs.get((perm_name, view_name)) or self.add_permission_view_menu(
-                    perm_name, view_name
+            for action_name, resource_name in perms:
+                perm = non_dag_perms.get((action_name, resource_name)) or self.create_permission(
+                    action_name, resource_name
                 )
 
-                if perm_view not in role.permissions:
-                    self.add_permission_role(role, perm_view)
-
-    def add_permissions(self, role, perms):
-        """Adds resource permissions to a given role."""
-        for perm_name, view_name in perms:
-            perm_view = self.add_permission_view_menu(perm_name, view_name)
-            self.add_permission_role(role, perm_view)
+                if perm not in role.permissions:
+                    self.add_permission_to_role(role, perm)
 
     def delete_role(self, role_name):
         """
@@ -242,7 +249,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         :param role_name: the name of a role in the ab_role table
         """
         session = self.get_session
-        role = session.query(sqla_models.Role).filter(sqla_models.Role.name == role_name).first()
+        role = session.query(Role).filter(Role.name == role_name).first()
         if role:
             self.log.info("Deleting role '%s'", role_name)
             session.delete(role)
@@ -260,55 +267,63 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         """
         if user is None:
             user = g.user
-        if user.is_anonymous:
-            public_role = current_app.appbuilder.get_app.config["AUTH_ROLE_PUBLIC"]
-            return [current_app.appbuilder.sm.find_role(public_role)] if public_role else []
         return user.roles
 
-    def get_current_user_permissions(self):
-        """Returns permissions for logged in user as a set of tuples with the perm name and view menu name"""
-        perms_views = set()
-        for role in self.get_user_roles():
-            perms_views.update(
-                {(perm_view.permission.name, perm_view.view_menu.name) for perm_view in role.permissions}
-            )
-        return perms_views
-
     def get_readable_dags(self, user):
         """Gets the DAGs readable by authenticated user."""
-        return self.get_accessible_dags([permissions.ACTION_CAN_READ], user)
+        warnings.warn(
+            "`get_readable_dags` has been deprecated. Please use `get_readable_dag_ids` instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", DeprecationWarning)
+            return self.get_accessible_dags([permissions.ACTION_CAN_READ], user)
 
     def get_editable_dags(self, user):
         """Gets the DAGs editable by authenticated user."""
-        return self.get_accessible_dags([permissions.ACTION_CAN_EDIT], user)
-
-    def get_readable_dag_ids(self, user) -> Set[str]:
-        """Gets the DAG IDs readable by authenticated user."""
-        return {dag.dag_id for dag in self.get_readable_dags(user)}
-
-    def get_editable_dag_ids(self, user) -> Set[str]:
-        """Gets the DAG IDs editable by authenticated user."""
-        return {dag.dag_id for dag in self.get_editable_dags(user)}
-
-    def get_accessible_dag_ids(self, user) -> Set[str]:
-        """Gets the DAG IDs editable or readable by authenticated user."""
-        accessible_dags = self.get_accessible_dags(
-            [permissions.ACTION_CAN_EDIT, permissions.ACTION_CAN_READ], user
+        warnings.warn(
+            "`get_editable_dags` has been deprecated. Please use `get_editable_dag_ids` instead.",
+            DeprecationWarning,
+            stacklevel=2,
         )
-        return {dag.dag_id for dag in accessible_dags}
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", DeprecationWarning)
+            return self.get_accessible_dags([permissions.ACTION_CAN_EDIT], user)
 
     @provide_session
     def get_accessible_dags(self, user_actions, user, session=None):
+        warnings.warn(
+            "`get_accessible_dags` has been deprecated. Please use `get_accessible_dag_ids` instead.",
+            DeprecationWarning,
+            stacklevel=3,
+        )
+        dag_ids = self.get_accessible_dag_ids(user, user_actions, session)
+        return session.query(DagModel).filter(DagModel.dag_id.in_(dag_ids))
+
+    def get_readable_dag_ids(self, user) -> Set[str]:
+        """Gets the DAG IDs readable by authenticated user."""
+        return self.get_accessible_dag_ids(user, [permissions.ACTION_CAN_READ])
+
+    def get_editable_dag_ids(self, user) -> Set[str]:
+        """Gets the DAG IDs editable by authenticated user."""
+        return self.get_accessible_dag_ids(user, [permissions.ACTION_CAN_EDIT])
+
+    @provide_session
+    def get_accessible_dag_ids(self, user, user_actions=None, session=None) -> Set[str]:
         """Generic function to get readable or writable DAGs for user."""
+        if not user_actions:
+            user_actions = [permissions.ACTION_CAN_EDIT, permissions.ACTION_CAN_READ]
+
         if user.is_anonymous:
-            roles = self.get_user_roles(user)
+            roles = user.roles
         else:
             user_query = (
                 session.query(User)
                 .options(
                     joinedload(User.roles)
                     .subqueryload(Role.permissions)
-                    .options(joinedload(PermissionView.permission), joinedload(PermissionView.view_menu))
+                    .options(joinedload(Permission.action), joinedload(Permission.resource))
                 )
                 .filter(User.id == user.id)
                 .first()
@@ -318,49 +333,48 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         resources = set()
         for role in roles:
             for permission in role.permissions:
-                action = permission.permission.name
+                action = permission.action.name
                 if action not in user_actions:
                     continue
 
-                resource = permission.view_menu.name
+                resource = permission.resource.name
                 if resource == permissions.RESOURCE_DAG:
-                    return session.query(DagModel)
+                    return {dag.dag_id for dag in session.query(DagModel.dag_id)}
 
                 if resource.startswith(permissions.RESOURCE_DAG_PREFIX):
                     resources.add(resource[len(permissions.RESOURCE_DAG_PREFIX) :])
                 else:
                     resources.add(resource)
-
-        return session.query(DagModel).filter(DagModel.dag_id.in_(resources))
+        return {dag.dag_id for dag in session.query(DagModel.dag_id).filter(DagModel.dag_id.in_(resources))}
 
     def can_access_some_dags(self, action: str, dag_id: Optional[str] = None) -> bool:
         """Checks if user has read or write access to some dags."""
         if dag_id and dag_id != '~':
-            return self.has_access(action, permissions.resource_name_for_dag(dag_id))
+            root_dag_id = self._get_root_dag_id(dag_id)
+            return self.has_access(action, permissions.resource_name_for_dag(root_dag_id))
 
         user = g.user
         if action == permissions.ACTION_CAN_READ:
-            return any(self.get_readable_dags(user))
-        return any(self.get_editable_dags(user))
+            return any(self.get_readable_dag_ids(user))
+        return any(self.get_editable_dag_ids(user))
 
     def can_read_dag(self, dag_id, user=None) -> bool:
         """Determines whether a user has DAG read access."""
-        if not user:
-            user = g.user
-        dag_resource_name = permissions.resource_name_for_dag(dag_id)
-        return self._has_view_access(
-            user, permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG
-        ) or self._has_view_access(user, permissions.ACTION_CAN_READ, dag_resource_name)
+        root_dag_id = self._get_root_dag_id(dag_id)
+        dag_resource_name = permissions.resource_name_for_dag(root_dag_id)
+        return self.has_access(permissions.ACTION_CAN_READ, dag_resource_name, user=user)
 
     def can_edit_dag(self, dag_id, user=None) -> bool:
         """Determines whether a user has DAG edit access."""
-        if not user:
-            user = g.user
-        dag_resource_name = permissions.resource_name_for_dag(dag_id)
+        root_dag_id = self._get_root_dag_id(dag_id)
+        dag_resource_name = permissions.resource_name_for_dag(root_dag_id)
+        return self.has_access(permissions.ACTION_CAN_EDIT, dag_resource_name, user=user)
 
-        return self._has_view_access(
-            user, permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG
-        ) or self._has_view_access(user, permissions.ACTION_CAN_EDIT, dag_resource_name)
+    def can_delete_dag(self, dag_id, user=None) -> bool:
+        """Determines whether a user has DAG delete access."""
+        root_dag_id = self._get_root_dag_id(dag_id)
+        dag_resource_name = permissions.resource_name_for_dag(root_dag_id)
+        return self.has_access(permissions.ACTION_CAN_DELETE, dag_resource_name, user=user)
 
     def prefixed_dag_id(self, dag_id):
         """Returns the permission name for a DAG id."""
@@ -370,85 +384,67 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
             DeprecationWarning,
             stacklevel=2,
         )
-        return permissions.resource_name_for_dag(dag_id)
+        root_dag_id = self._get_root_dag_id(dag_id)
+        return permissions.resource_name_for_dag(root_dag_id)
 
     def is_dag_resource(self, resource_name):
-        """Determines if a permission belongs to a DAG or all DAGs."""
+        """Determines if a resource belongs to a DAG or all DAGs."""
         if resource_name == permissions.RESOURCE_DAG:
             return True
         return resource_name.startswith(permissions.RESOURCE_DAG_PREFIX)
 
-    def has_access(self, permission, resource, user=None) -> bool:
+    def has_access(self, action_name, resource_name, user=None) -> bool:
         """
-        Verify whether a given user could perform certain permission
-        (e.g can_read, can_write) on the given resource.
+        Verify whether a given user could perform a certain action
+        (e.g can_read, can_write, can_delete) on the given resource.
 
-        :param permission: permission on resource (e.g can_read, can_edit).
-        :type permission: str
-        :param resource: name of view-menu or resource.
-        :type resource: str
+        :param action_name: action_name on resource (e.g can_read, can_edit).
+        :param resource_name: name of view-menu or resource.
         :param user: user name
-        :type user: str
-        :return: a bool whether user could perform certain permission on the resource.
+        :return: Whether user could perform certain action on the resource.
         :rtype bool
         """
         if not user:
             user = g.user
+        if (action_name, resource_name) in user.perms:
+            return True
 
-        if user.is_anonymous:
-            user.roles = self.get_user_roles(user)
+        if self.is_dag_resource(resource_name):
+            if (action_name, permissions.RESOURCE_DAG) in user.perms:
+                return True
+            return (action_name, resource_name) in user.perms
 
-        has_access = self._has_view_access(user, permission, resource)
-        # FAB built-in view access method. Won't work for AllDag access.
+        return False
 
-        if self.is_dag_resource(resource):
-            if permission == permissions.ACTION_CAN_READ:
-                has_access |= self.can_read_dag(resource, user)
-            elif permission == permissions.ACTION_CAN_EDIT:
-                has_access |= self.can_edit_dag(resource, user)
-
-        return has_access
-
-    def _get_and_cache_perms(self):
-        """Cache permissions-views"""
-        self.perms = self.get_current_user_permissions()
-
-    def _has_role(self, role_name_or_list):
+    def _has_role(self, role_name_or_list, user):
         """Whether the user has this role name"""
         if not isinstance(role_name_or_list, list):
             role_name_or_list = [role_name_or_list]
-        return any(r.name in role_name_or_list for r in self.get_user_roles())
+        return any(r.name in role_name_or_list for r in user.roles)
 
-    def _has_perm(self, permission_name, view_menu_name):
-        """Whether the user has this perm"""
-        if hasattr(self, 'perms') and self.perms is not None:
-            if (permission_name, view_menu_name) in self.perms:
-                return True
-        # rebuild the permissions set
-        self._get_and_cache_perms()
-        return (permission_name, view_menu_name) in self.perms
-
-    def has_all_dags_access(self):
+    def has_all_dags_access(self, user):
         """
         Has all the dag access in any of the 3 cases:
         1. Role needs to be in (Admin, Viewer, User, Op).
-        2. Has can_read permission on dags view.
-        3. Has can_edit permission on dags view.
+        2. Has can_read action on dags resource.
+        3. Has can_edit action on dags resource.
         """
+        if not user:
+            user = g.user
         return (
-            self._has_role(['Admin', 'Viewer', 'Op', 'User'])
-            or self._has_perm(permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG)
-            or self._has_perm(permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG)
+            self._has_role(['Admin', 'Viewer', 'Op', 'User'], user)
+            or self.has_access(permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG, user)
+            or self.has_access(permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG, user)
         )
 
     def clean_perms(self):
         """FAB leaves faulty permissions that need to be cleaned up"""
         self.log.debug('Cleaning faulty perms')
         sesh = self.get_session
-        pvms = sesh.query(sqla_models.PermissionView).filter(
+        perms = sesh.query(Permission).filter(
             or_(
-                sqla_models.PermissionView.permission == None,  # noqa pylint: disable=singleton-comparison
-                sqla_models.PermissionView.view_menu == None,  # noqa pylint: disable=singleton-comparison
+                Permission.action == None,  # noqa
+                Permission.resource == None,  # noqa
             )
         )
         # Since FAB doesn't define ON DELETE CASCADE on these tables, we need
@@ -456,36 +452,33 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         # relationship object too. :(
 
         deleted_count = 0
-        for pvm in pvms:
-            sesh.delete(pvm)
+        for perm in perms:
+            sesh.delete(perm)
             deleted_count += 1
         sesh.commit()
         if deleted_count:
             self.log.info('Deleted %s faulty permissions', deleted_count)
 
-    def _merge_perm(self, permission_name, view_menu_name):
+    def _merge_perm(self, action_name, resource_name):
         """
-        Add the new (permission, view_menu) to assoc_permissionview_role if it doesn't exist.
-        It will add the related entry to ab_permission
-        and ab_view_menu two meta tables as well.
+        Add the new (action, resource) to assoc_permission_role if it doesn't exist.
+        It will add the related entry to ab_permission and ab_resource two meta tables as well.
 
-        :param permission_name: Name of the permission.
-        :type permission_name: str
-        :param view_menu_name: Name of the view-menu
-        :type view_menu_name: str
+        :param action_name: Name of the action
+        :param resource_name: Name of the resource
         :return:
         """
-        permission = self.find_permission(permission_name)
-        view_menu = self.find_view_menu(view_menu_name)
-        permission_view = None
-        if permission and view_menu:
-            permission_view = (
-                self.get_session.query(self.permissionview_model)
-                .filter_by(permission=permission, view_menu=view_menu)
+        action = self.get_action(action_name)
+        resource = self.get_resource(resource_name)
+        perm = None
+        if action and resource:
+            perm = (
+                self.get_session.query(self.permission_model)
+                .filter_by(action=action, resource=resource)
                 .first()
             )
-        if not permission_view and permission_name and view_menu_name:
-            self.add_permission_view_menu(permission_name, view_menu_name)
+        if not perm and action_name and resource_name:
+            self.create_permission(action_name, resource_name)
 
     def add_homepage_access_to_custom_roles(self):
         """
@@ -493,46 +486,42 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
 
         :return: None.
         """
-        website_permission = self.add_permission_view_menu(
-            permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE
-        )
+        website_permission = self.create_permission(permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE)
         custom_roles = [role for role in self.get_all_roles() if role.name not in EXISTING_ROLES]
         for role in custom_roles:
-            self.add_permission_role(role, website_permission)
+            self.add_permission_to_role(role, website_permission)
 
         self.get_session.commit()
 
     def get_all_permissions(self) -> Set[Tuple[str, str]]:
-        """Returns all permissions as a set of tuples with the perm name and view menu name"""
+        """Returns all permissions as a set of tuples with the action and resource names"""
         return set(
-            self.get_session.query(self.permissionview_model)
-            .join(self.permission_model)
-            .join(self.viewmenu_model)
-            .with_entities(self.permission_model.name, self.viewmenu_model.name)
+            self.get_session.query(self.permission_model)
+            .join(self.permission_model.action)
+            .join(self.permission_model.resource)
+            .with_entities(self.action_model.name, self.resource_model.name)
             .all()
         )
 
-    def _get_all_non_dag_permissionviews(self) -> Dict[Tuple[str, str], PermissionView]:
+    def _get_all_non_dag_permissions(self) -> Dict[Tuple[str, str], Permission]:
         """
-        Returns a dict with a key of (perm name, view menu name) and value of perm view
-        with all perm views except those that are for specific DAGs.
+        Returns a dict with a key of (action_name, resource_name) and value of permission
+        with all permissions except those that are for specific DAGs.
         """
         return {
-            (perm_name, viewmodel_name): viewmodel
-            for perm_name, viewmodel_name, viewmodel in (
-                self.get_session.query(self.permissionview_model)
-                .join(self.permission_model)
-                .join(self.viewmenu_model)
-                .filter(~self.viewmenu_model.name.like(f"{permissions.RESOURCE_DAG_PREFIX}%"))
-                .with_entities(
-                    self.permission_model.name, self.viewmenu_model.name, self.permissionview_model
-                )
+            (action_name, resource_name): viewmodel
+            for action_name, resource_name, viewmodel in (
+                self.get_session.query(self.permission_model)
+                .join(self.permission_model.action)
+                .join(self.permission_model.resource)
+                .filter(~self.resource_model.name.like(f"{permissions.RESOURCE_DAG_PREFIX}%"))
+                .with_entities(self.action_model.name, self.resource_model.name, self.permission_model)
                 .all()
             )
         }
 
     def _get_all_roles_with_permissions(self) -> Dict[str, Role]:
-        """Returns a dict with a key of role name and value of role with eagrly loaded permissions"""
+        """Returns a dict with a key of role name and value of role with early loaded permissions"""
         return {
             r.name: r
             for r in (
@@ -542,8 +531,8 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
 
     def create_dag_specific_permissions(self) -> None:
         """
-        Creates 'can_read' and 'can_edit' permissions for all DAGs,
-        along with any `access_control` permissions provided in them.
+        Creates 'can_read', 'can_edit', and 'can_delete' permissions for all
+        DAGs, along with any `access_control` permissions provided in them.
 
         This does iterate through ALL the DAGs, which can be slow. See `sync_perm_for_dag`
         if you only need to sync a single DAG.
@@ -556,38 +545,35 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         dags = dagbag.dags.values()
 
         for dag in dags:
-            dag_resource_name = permissions.resource_name_for_dag(dag.dag_id)
-            for perm_name in self.DAG_PERMS:
-                if (perm_name, dag_resource_name) not in perms:
-                    self._merge_perm(perm_name, dag_resource_name)
+            root_dag_id = dag.parent_dag.dag_id if dag.parent_dag else dag.dag_id
+            dag_resource_name = permissions.resource_name_for_dag(root_dag_id)
+            for action_name in self.DAG_ACTIONS:
+                if (action_name, dag_resource_name) not in perms:
+                    self._merge_perm(action_name, dag_resource_name)
 
             if dag.access_control:
-                self._sync_dag_view_permissions(dag_resource_name, dag.access_control)
+                self.sync_perm_for_dag(dag_resource_name, dag.access_control)
 
-    def update_admin_perm_view(self):
+    def update_admin_permission(self):
         """
-        Admin should have all the permission-views, except the dag views.
+        Admin should have all the permissions, except the dag permissions.
         because Admin already has Dags permission.
         Add the missing ones to the table for admin.
 
         :return: None.
         """
-        dag_pvs = (
-            self.get_session.query(sqla_models.ViewMenu)
-            .filter(sqla_models.ViewMenu.name.like(f"{permissions.RESOURCE_DAG_PREFIX}%"))
-            .all()
-        )
-        pv_ids = [pv.id for pv in dag_pvs]
-        pvms = (
-            self.get_session.query(sqla_models.PermissionView)
-            .filter(~sqla_models.PermissionView.view_menu_id.in_(pv_ids))
+        dag_resources = (
+            self.get_session.query(Resource)
+            .filter(Resource.name.like(f"{permissions.RESOURCE_DAG_PREFIX}%"))
             .all()
         )
+        resource_ids = [resource.id for resource in dag_resources]
+        perms = self.get_session.query(Permission).filter(~Permission.resource_id.in_(resource_ids)).all()
 
-        pvms = [p for p in pvms if p.permission and p.view_menu]
+        perms = [p for p in perms if p.action and p.resource]
 
         admin = self.find_role('Admin')
-        admin.permissions = list(set(admin.permissions) | set(pvms))
+        admin.permissions = list(set(admin.permissions) | set(perms))
 
         self.get_session.commit()
 
@@ -599,7 +585,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
 
         :return: None.
         """
-        # Create global all-dag VM
+        # Create global all-dag permissions
         self.create_perm_vm_for_all_dag()
 
         # Sync the default roles (Admin, Viewer, User, Op, public) with related permissions
@@ -607,7 +593,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
 
         self.add_homepage_access_to_custom_roles()
         # init existing roles, the rest role could be created through UI.
-        self.update_admin_perm_view()
+        self.update_admin_permission()
         self.clean_perms()
 
     def sync_resource_permissions(self, perms=None):
@@ -615,9 +601,9 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         if not perms:
             return
 
-        for action, resource in perms:
-            self.add_view_menu(resource)
-            self.add_permission_view_menu(action, resource)
+        for action_name, resource_name in perms:
+            self.create_resource(resource_name)
+            self.create_permission(action_name, resource_name)
 
     def sync_perm_for_dag(self, dag_id, access_control=None):
         """
@@ -625,86 +611,84 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
         as only / refresh button or DagBag will call this function
 
         :param dag_id: the ID of the DAG whose permissions should be updated
-        :type dag_id: str
         :param access_control: a dict where each key is a rolename and
-            each value is a set() of permission names (e.g.,
+            each value is a set() of action names (e.g.,
             {'can_read'}
-        :type access_control: dict
         :return:
         """
         dag_resource_name = permissions.resource_name_for_dag(dag_id)
-        for dag_perm in self.DAG_PERMS:
-            self.add_permission_view_menu(dag_perm, dag_resource_name)
+        for dag_action_name in self.DAG_ACTIONS:
+            self.create_permission(dag_action_name, dag_resource_name)
 
         if access_control:
             self._sync_dag_view_permissions(dag_resource_name, access_control)
 
     def _sync_dag_view_permissions(self, dag_id, access_control):
-        """Set the access policy on the given DAG's ViewModel.
+        """
+        Set the access policy on the given DAG's ViewModel.
 
         :param dag_id: the ID of the DAG whose permissions should be updated
-        :type dag_id: str
         :param access_control: a dict where each key is a rolename and
-            each value is a set() of permission names (e.g.,
-            {'can_read'}
-        :type access_control: dict
+            each value is a set() of action names (e.g. {'can_read'})
         """
+
         dag_resource_name = permissions.resource_name_for_dag(dag_id)
 
-        def _get_or_create_dag_permission(perm_name):
-            dag_perm = self.find_permission_view_menu(perm_name, dag_resource_name)
-            if not dag_perm:
-                self.log.info("Creating new permission '%s' on view '%s'", perm_name, dag_resource_name)
-                dag_perm = self.add_permission_view_menu(perm_name, dag_resource_name)
+        def _get_or_create_dag_permission(action_name: str) -> Optional[Permission]:
+            perm = self.get_permission(action_name, dag_resource_name)
+            if not perm:
+                self.log.info("Creating new action '%s' on resource '%s'", action_name, dag_resource_name)
+                perm = self.create_permission(action_name, dag_resource_name)
 
-            return dag_perm
+            return perm
 
-        def _revoke_stale_permissions(dag_view):
-            existing_dag_perms = self.find_permissions_view_menu(dag_view)
+        def _revoke_stale_permissions(resource: Resource):
+            existing_dag_perms = self.get_resource_permissions(resource)
             for perm in existing_dag_perms:
                 non_admin_roles = [role for role in perm.role if role.name != 'Admin']
                 for role in non_admin_roles:
                     target_perms_for_role = access_control.get(role.name, {})
-                    if perm.permission.name not in target_perms_for_role:
+                    if perm.action.name not in target_perms_for_role:
                         self.log.info(
                             "Revoking '%s' on DAG '%s' for role '%s'",
-                            perm.permission,
+                            perm.action,
                             dag_resource_name,
                             role.name,
                         )
-                        self.del_permission_role(role, perm)
+                        self.remove_permission_from_role(role, perm)
 
-        dag_view = self.find_view_menu(dag_resource_name)
-        if dag_view:
-            _revoke_stale_permissions(dag_view)
+        resource = self.get_resource(dag_resource_name)
+        if resource:
+            _revoke_stale_permissions(resource)
 
-        for rolename, perms in access_control.items():
+        for rolename, action_names in access_control.items():
             role = self.find_role(rolename)
             if not role:
                 raise AirflowException(
-                    "The access_control mapping for DAG '{}' includes a role "
-                    "named '{}', but that role does not exist".format(dag_id, rolename)
+                    f"The access_control mapping for DAG '{dag_id}' includes a role named "
+                    f"'{rolename}', but that role does not exist"
                 )
 
-            perms = set(perms)
-            invalid_perms = perms - self.DAG_PERMS
-            if invalid_perms:
+            action_names = set(action_names)
+            invalid_action_names = action_names - self.DAG_ACTIONS
+            if invalid_action_names:
                 raise AirflowException(
-                    "The access_control map for DAG '{}' includes the following "
-                    "invalid permissions: {}; The set of valid permissions "
-                    "is: {}".format(dag_resource_name, invalid_perms, self.DAG_PERMS)
+                    f"The access_control map for DAG '{dag_resource_name}' includes "
+                    f"the following invalid permissions: {invalid_action_names}; "
+                    f"The set of valid permissions is: {self.DAG_ACTIONS}"
                 )
 
-            for perm_name in perms:
-                dag_perm = _get_or_create_dag_permission(perm_name)
-                self.add_permission_role(role, dag_perm)
+            for action_name in action_names:
+                dag_perm = _get_or_create_dag_permission(action_name)
+                if dag_perm:
+                    self.add_permission_to_role(role, dag_perm)
 
     def create_perm_vm_for_all_dag(self):
         """Create perm-vm if not exist and insert into FAB security model for all-dags."""
         # create perm for global logical dag
-        for dag_vm in self.DAG_VMS:
-            for perm in self.DAG_PERMS:
-                self._merge_perm(permission_name=perm, view_menu_name=dag_vm)
+        for resource_name in self.DAG_RESOURCES:
+            for action_name in self.DAG_ACTIONS:
+                self._merge_perm(action_name, resource_name)
 
     def check_authorization(
         self, perms: Optional[Sequence[Tuple[str, str]]] = None, dag_id: Optional[str] = None
@@ -717,6 +701,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
             if perm in (
                 (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
                 (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
+                (permissions.ACTION_CAN_DELETE, permissions.RESOURCE_DAG),
             ):
                 can_access_all_dags = self.has_access(*perm)
                 if can_access_all_dags:
@@ -736,7 +721,7 @@ class AirflowSecurityManager(SecurityManager, LoggingMixin):  # pylint: disable=
 class ApplessAirflowSecurityManager(AirflowSecurityManager):
     """Security Manager that doesn't need the whole flask app"""
 
-    def __init__(self, session=None):  # pylint: disable=super-init-not-called
+    def __init__(self, session=None):
         self.session = session
 
     @property
